# flake8: noqa

# import apis into api package
from crypticorn.notification.client.api.notifications_api import NotificationsApi
from crypticorn.notification.client.api.settings_api import SettingsApi
from crypticorn.notification.client.api.status_api import StatusApi
from crypticorn.notification.client.api.templates_api import TemplatesApi



# coding: utf-8

"""
DEX AI API

API for DEX AI

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set, Union

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing_extensions import Self


class SignalOverviewStats(BaseModel):
    """
    Model for signal statistics response
    """  # noqa: E501

    timestamp: StrictInt = Field(
        description="The unix timestamp of the stats calculation"
    )
    total: StrictInt = Field(description="Total number of tokens analyzed")
    win_rate: Union[StrictFloat, StrictInt] = Field(
        description="Overall win rate as a percentage"
    )
    avg_performance: Union[StrictFloat, StrictInt] = Field(
        description="Average performance as a percentage"
    )
    median_performance: Union[StrictFloat, StrictInt] = Field(
        description="Median performance as a percentage"
    )
    best_risk_category: Optional[StrictStr] = None
    best_month: Optional[StrictStr] = None
    best_holding_period: Optional[StrictStr] = None
    best_launch_hour: Optional[StrictInt] = None
    best_launch_day: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = [
        "timestamp",
        "total",
        "win_rate",
        "avg_performance",
        "median_performance",
        "best_risk_category",
        "best_month",
        "best_holding_period",
        "best_launch_hour",
        "best_launch_day",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SignalOverviewStats from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if best_risk_category (nullable) is None
        # and model_fields_set contains the field
        if (
            self.best_risk_category is None
            and "best_risk_category" in self.model_fields_set
        ):
            _dict["best_risk_category"] = None

        # set to None if best_month (nullable) is None
        # and model_fields_set contains the field
        if self.best_month is None and "best_month" in self.model_fields_set:
            _dict["best_month"] = None

        # set to None if best_holding_period (nullable) is None
        # and model_fields_set contains the field
        if (
            self.best_holding_period is None
            and "best_holding_period" in self.model_fields_set
        ):
            _dict["best_holding_period"] = None

        # set to None if best_launch_hour (nullable) is None
        # and model_fields_set contains the field
        if (
            self.best_launch_hour is None
            and "best_launch_hour" in self.model_fields_set
        ):
            _dict["best_launch_hour"] = None

        # set to None if best_launch_day (nullable) is None
        # and model_fields_set contains the field
        if self.best_launch_day is None and "best_launch_day" in self.model_fields_set:
            _dict["best_launch_day"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SignalOverviewStats from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "timestamp": obj.get("timestamp"),
                "total": obj.get("total"),
                "win_rate": obj.get("win_rate"),
                "avg_performance": obj.get("avg_performance"),
                "median_performance": obj.get("median_performance"),
                "best_risk_category": obj.get("best_risk_category"),
                "best_month": obj.get("best_month"),
                "best_holding_period": obj.get("best_holding_period"),
                "best_launch_hour": obj.get("best_launch_hour"),
                "best_launch_day": obj.get("best_launch_day"),
            }
        )
        return _obj



# coding: utf-8

# flake8: noqa
"""
DEX AI API

API for DEX AI

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from crypticorn.dex.client.models.api_error_identifier import ApiErrorIdentifier
from crypticorn.dex.client.models.api_error_level import ApiErrorLevel
from crypticorn.dex.client.models.api_error_type import ApiErrorType
from crypticorn.dex.client.models.exception_detail import ExceptionDetail
from crypticorn.dex.client.models.paginated_response_signal_with_token import (
    PaginatedResponseSignalWithToken,
)
from crypticorn.dex.client.models.risk import Risk
from crypticorn.dex.client.models.signal_overview_stats import SignalOverviewStats
from crypticorn.dex.client.models.signal_volume import SignalVolume
from crypticorn.dex.client.models.signal_with_token import SignalWithToken
from crypticorn.dex.client.models.token_data import TokenData
from crypticorn.dex.client.models.token_detail import TokenDetail



# coding: utf-8

"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 2.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    StrictBool,
    StrictInt,
    StrictStr,
    field_validator,
)
from typing_extensions import Self

from crypticorn.trade.client.models.trading_action_type import TradingActionType


class Order(BaseModel):
    """
    Response model for orders. This is the model that is returned by the API and the database.
    """  # noqa: E501

    created_at: StrictInt = Field(description="Timestamp of creation")
    updated_at: StrictInt = Field(description="Timestamp of last update")
    id: StrictStr = Field(description="Unique identifier for the resource")
    exchange: StrictStr = Field(description="Exchange name. Of type Exchange")
    symbol: StrictStr = Field(description="Trading symbol on exchange")
    action_type: TradingActionType = Field(
        description="Type of trading action. Of type TradingActionType"
    )
    status: StrictStr = Field(
        description="Trade status of the order. Of type OrderStatus"
    )
    market_type: StrictStr = Field(
        description="Market type of the order. Of type MarketType"
    )
    trading_action_id: Optional[StrictStr] = None
    execution_id: Optional[StrictStr] = None
    exchange_order_id: Optional[StrictStr] = None
    position_id: Optional[StrictStr] = None
    api_key_id: Optional[StrictStr] = None
    bot_id: Optional[StrictStr] = None
    client_order_id: Optional[StrictStr] = None
    common_symbol: Optional[StrictStr] = None
    price: StrictStr = Field(description="Price of the order")
    margin_mode: Optional[StrictStr] = None
    status_code: StrictStr = Field(description="API error identifier")
    filled_perc: Optional[StrictStr] = Field(
        default="0", description="Percentage of the order filled"
    )
    filled_qty: Optional[StrictStr] = Field(
        default="0",
        description="Quantity filled. Needed for pnl calculation. In the symbol's base currency.",
    )
    sent_qty: Optional[StrictStr] = Field(
        default="0",
        description="Quantity sent to the exchange. In the symbol's base currency.",
    )
    fee: Optional[StrictStr] = None
    leverage: Optional[StrictInt] = None
    order_details: Optional[Dict[str, Any]] = Field(
        default=None, description="Exchange specific details of the order"
    )
    pnl: Optional[StrictStr] = None
    order_time: Optional[StrictInt] = None
    is_lost: Optional[StrictBool] = None
    __properties: ClassVar[List[str]] = [
        "created_at",
        "updated_at",
        "id",
        "exchange",
        "symbol",
        "action_type",
        "status",
        "market_type",
        "trading_action_id",
        "execution_id",
        "exchange_order_id",
        "position_id",
        "api_key_id",
        "bot_id",
        "client_order_id",
        "common_symbol",
        "price",
        "margin_mode",
        "status_code",
        "filled_perc",
        "filled_qty",
        "sent_qty",
        "fee",
        "leverage",
        "order_details",
        "pnl",
        "order_time",
        "is_lost",
    ]

    @field_validator("exchange")
    def exchange_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(["hyperliquid"]):
            raise ValueError("must be one of enum values ('hyperliquid')")
        return value

    @field_validator("status")
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(
            ["new", "filled", "partially_filled", "cancelled", "failed"]
        ):
            raise ValueError(
                "must be one of enum values ('new', 'filled', 'partially_filled', 'cancelled', 'failed')"
            )
        return value

    @field_validator("market_type")
    def market_type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(["spot", "futures"]):
            raise ValueError("must be one of enum values ('spot', 'futures')")
        return value

    @field_validator("margin_mode")
    def margin_mode_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(["isolated", "cross"]):
            raise ValueError("must be one of enum values ('isolated', 'cross')")
        return value

    @field_validator("status_code")
    def status_code_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(
            [
                "unknown_error",
                "invalid_data_request",
                "object_not_found",
                "forbidden",
                "exchange_key_already_exists",
                "exchange_key_in_use",
                "order_size_too_small",
                "order_size_too_large",
                "bot_stopping_completed",
                "success",
                "strategy_disabled",
                "bot_already_deleted",
                "object_already_exists",
                "invalid_exchange_key",
                "hedge_mode_not_active",
                "strategy_not_supporting_exchange",
                "margin_mode_clash",
                "order_price_invalid",
                "insufficient_margin",
                "insufficient_balance",
                "exchange_invalid_parameter",
                "exchange_invalid_signature",
                "exchange_rate_limit",
                "order_not_found",
                "symbol_not_found",
                "order_limit_exceeded",
                "exchange_system_error",
                "exchange_system_busy",
                "rpc_timeout",
                "position_not_found",
                "leverage_exceeded",
                "trading_suspended",
                "exchange_maintenance",
                "exchange_ip_restricted",
                "post_only_rejected",
                "content_type_error",
                "exchange_http_error",
                "orphan_close_order",
                "cancelled_open_order",
                "failed_open_order",
                "orphan_open_order",
                "trading_action_skipped_bot_stopping",
                "trading_action_expired",
                "black_swan",
            ]
        ):
            raise ValueError(
                "must be one of enum values ('unknown_error', 'invalid_data_request', 'object_not_found', 'forbidden', 'exchange_key_already_exists', 'exchange_key_in_use', 'order_size_too_small', 'order_size_too_large', 'bot_stopping_completed', 'success', 'strategy_disabled', 'bot_already_deleted', 'object_already_exists', 'invalid_exchange_key', 'hedge_mode_not_active', 'strategy_not_supporting_exchange', 'margin_mode_clash', 'order_price_invalid', 'insufficient_margin', 'insufficient_balance', 'exchange_invalid_parameter', 'exchange_invalid_signature', 'exchange_rate_limit', 'order_not_found', 'symbol_not_found', 'order_limit_exceeded', 'exchange_system_error', 'exchange_system_busy', 'rpc_timeout', 'position_not_found', 'leverage_exceeded', 'trading_suspended', 'exchange_maintenance', 'exchange_ip_restricted', 'post_only_rejected', 'content_type_error', 'exchange_http_error', 'orphan_close_order', 'cancelled_open_order', 'failed_open_order', 'orphan_open_order', 'trading_action_skipped_bot_stopping', 'trading_action_expired', 'black_swan')"
            )
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Order from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if trading_action_id (nullable) is None
        # and model_fields_set contains the field
        if (
            self.trading_action_id is None
            and "trading_action_id" in self.model_fields_set
        ):
            _dict["trading_action_id"] = None

        # set to None if execution_id (nullable) is None
        # and model_fields_set contains the field
        if self.execution_id is None and "execution_id" in self.model_fields_set:
            _dict["execution_id"] = None

        # set to None if exchange_order_id (nullable) is None
        # and model_fields_set contains the field
        if (
            self.exchange_order_id is None
            and "exchange_order_id" in self.model_fields_set
        ):
            _dict["exchange_order_id"] = None

        # set to None if position_id (nullable) is None
        # and model_fields_set contains the field
        if self.position_id is None and "position_id" in self.model_fields_set:
            _dict["position_id"] = None

        # set to None if api_key_id (nullable) is None
        # and model_fields_set contains the field
        if self.api_key_id is None and "api_key_id" in self.model_fields_set:
            _dict["api_key_id"] = None

        # set to None if bot_id (nullable) is None
        # and model_fields_set contains the field
        if self.bot_id is None and "bot_id" in self.model_fields_set:
            _dict["bot_id"] = None

        # set to None if client_order_id (nullable) is None
        # and model_fields_set contains the field
        if self.client_order_id is None and "client_order_id" in self.model_fields_set:
            _dict["client_order_id"] = None

        # set to None if common_symbol (nullable) is None
        # and model_fields_set contains the field
        if self.common_symbol is None and "common_symbol" in self.model_fields_set:
            _dict["common_symbol"] = None

        # set to None if margin_mode (nullable) is None
        # and model_fields_set contains the field
        if self.margin_mode is None and "margin_mode" in self.model_fields_set:
            _dict["margin_mode"] = None

        # set to None if fee (nullable) is None
        # and model_fields_set contains the field
        if self.fee is None and "fee" in self.model_fields_set:
            _dict["fee"] = None

        # set to None if leverage (nullable) is None
        # and model_fields_set contains the field
        if self.leverage is None and "leverage" in self.model_fields_set:
            _dict["leverage"] = None

        # set to None if pnl (nullable) is None
        # and model_fields_set contains the field
        if self.pnl is None and "pnl" in self.model_fields_set:
            _dict["pnl"] = None

        # set to None if order_time (nullable) is None
        # and model_fields_set contains the field
        if self.order_time is None and "order_time" in self.model_fields_set:
            _dict["order_time"] = None

        # set to None if is_lost (nullable) is None
        # and model_fields_set contains the field
        if self.is_lost is None and "is_lost" in self.model_fields_set:
            _dict["is_lost"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Order from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "created_at": obj.get("created_at"),
                "updated_at": obj.get("updated_at"),
                "id": obj.get("id"),
                "exchange": obj.get("exchange"),
                "symbol": obj.get("symbol"),
                "action_type": obj.get("action_type"),
                "status": obj.get("status"),
                "market_type": obj.get("market_type"),
                "trading_action_id": obj.get("trading_action_id"),
                "execution_id": obj.get("execution_id"),
                "exchange_order_id": obj.get("exchange_order_id"),
                "position_id": obj.get("position_id"),
                "api_key_id": obj.get("api_key_id"),
                "bot_id": obj.get("bot_id"),
                "client_order_id": obj.get("client_order_id"),
                "common_symbol": obj.get("common_symbol"),
                "price": obj.get("price"),
                "margin_mode": obj.get("margin_mode"),
                "status_code": obj.get("status_code"),
                "filled_perc": (
                    obj.get("filled_perc")
                    if obj.get("filled_perc") is not None
                    else "0"
                ),
                "filled_qty": (
                    obj.get("filled_qty") if obj.get("filled_qty") is not None else "0"
                ),
                "sent_qty": (
                    obj.get("sent_qty") if obj.get("sent_qty") is not None else "0"
                ),
                "fee": obj.get("fee"),
                "leverage": obj.get("leverage"),
                "order_details": obj.get("order_details"),
                "pnl": obj.get("pnl"),
                "order_time": obj.get("order_time"),
                "is_lost": obj.get("is_lost"),
            }
        )
        return _obj



# coding: utf-8

# flake8: noqa

"""
Hive AI API

API for Hive AI model training and evaluation

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# Define package exports
__all__ = [
    "DataApi",
    "ModelsApi",
    "StatusApi",
    "ApiResponse",
    "ApiClient",
    "Configuration",
    "OpenApiException",
    "ApiTypeError",
    "ApiValueError",
    "ApiKeyError",
    "ApiAttributeError",
    "ApiException",
    "ApiErrorIdentifier",
    "ApiErrorLevel",
    "ApiErrorType",
    "CoinInfo",
    "Coins",
    "DataDownloadResponse",
    "DataInfo",
    "DataOptions",
    "DataVersion",
    "DataVersionInfo",
    "DownloadLinks",
    "Evaluation",
    "EvaluationResponse",
    "ExceptionDetail",
    "FeatureSize",
    "ModelCreate",
    "ModelRead",
    "ModelStatus",
    "ModelUpdate",
    "Target",
    "TargetInfo",
    "TargetType",
]

# import apis into sdk package
from crypticorn.hive.client.api.data_api import DataApi as DataApi
from crypticorn.hive.client.api.models_api import ModelsApi as ModelsApi
from crypticorn.hive.client.api.status_api import StatusApi as StatusApi
from crypticorn.hive.client.api_client import ApiClient as ApiClient

# import ApiClient
from crypticorn.hive.client.api_response import ApiResponse as ApiResponse
from crypticorn.hive.client.configuration import Configuration as Configuration
from crypticorn.hive.client.exceptions import ApiAttributeError as ApiAttributeError
from crypticorn.hive.client.exceptions import ApiException as ApiException
from crypticorn.hive.client.exceptions import ApiKeyError as ApiKeyError
from crypticorn.hive.client.exceptions import ApiTypeError as ApiTypeError
from crypticorn.hive.client.exceptions import ApiValueError as ApiValueError
from crypticorn.hive.client.exceptions import OpenApiException as OpenApiException

# import models into sdk package
from crypticorn.hive.client.models.api_error_identifier import (
    ApiErrorIdentifier as ApiErrorIdentifier,
)
from crypticorn.hive.client.models.api_error_level import ApiErrorLevel as ApiErrorLevel
from crypticorn.hive.client.models.api_error_type import ApiErrorType as ApiErrorType
from crypticorn.hive.client.models.coin_info import CoinInfo as CoinInfo
from crypticorn.hive.client.models.coins import Coins as Coins
from crypticorn.hive.client.models.data_download_response import (
    DataDownloadResponse as DataDownloadResponse,
)
from crypticorn.hive.client.models.data_info import DataInfo as DataInfo
from crypticorn.hive.client.models.data_options import DataOptions as DataOptions
from crypticorn.hive.client.models.data_version import DataVersion as DataVersion
from crypticorn.hive.client.models.data_version_info import (
    DataVersionInfo as DataVersionInfo,
)
from crypticorn.hive.client.models.download_links import DownloadLinks as DownloadLinks
from crypticorn.hive.client.models.evaluation import Evaluation as Evaluation
from crypticorn.hive.client.models.evaluation_response import (
    EvaluationResponse as EvaluationResponse,
)
from crypticorn.hive.client.models.exception_detail import (
    ExceptionDetail as ExceptionDetail,
)
from crypticorn.hive.client.models.feature_size import FeatureSize as FeatureSize
from crypticorn.hive.client.models.model_create import ModelCreate as ModelCreate
from crypticorn.hive.client.models.model_read import ModelRead as ModelRead
from crypticorn.hive.client.models.model_status import ModelStatus as ModelStatus
from crypticorn.hive.client.models.model_update import ModelUpdate as ModelUpdate
from crypticorn.hive.client.models.target import Target as Target
from crypticorn.hive.client.models.target_info import TargetInfo as TargetInfo
from crypticorn.hive.client.models.target_type import TargetType as TargetType



# coding: utf-8

"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 2.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set, Union

from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    StrictBool,
    StrictStr,
    field_validator,
)
from typing_extensions import Annotated, Self


class StrategyUpdate(BaseModel):
    """
    Strategy model for API update operations. Fields cannot be unset.
    """  # noqa: E501

    name: Optional[StrictStr] = None
    description: Optional[StrictStr] = None
    exchanges: Optional[List[StrictStr]] = None
    enabled: Optional[StrictBool] = None
    performance_fee: Optional[
        Union[
            Annotated[float, Field(le=1.0, strict=True, ge=0.0)],
            Annotated[int, Field(le=1, strict=True, ge=0)],
        ]
    ] = None
    __properties: ClassVar[List[str]] = [
        "name",
        "description",
        "exchanges",
        "enabled",
        "performance_fee",
    ]

    @field_validator("exchanges")
    def exchanges_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        for i in value:
            if i not in set(["hyperliquid"]):
                raise ValueError("each list item must be one of ('hyperliquid')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of StrategyUpdate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict["name"] = None

        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict["description"] = None

        # set to None if exchanges (nullable) is None
        # and model_fields_set contains the field
        if self.exchanges is None and "exchanges" in self.model_fields_set:
            _dict["exchanges"] = None

        # set to None if enabled (nullable) is None
        # and model_fields_set contains the field
        if self.enabled is None and "enabled" in self.model_fields_set:
            _dict["enabled"] = None

        # set to None if performance_fee (nullable) is None
        # and model_fields_set contains the field
        if self.performance_fee is None and "performance_fee" in self.model_fields_set:
            _dict["performance_fee"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of StrategyUpdate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "name": obj.get("name"),
                "description": obj.get("description"),
                "exchanges": obj.get("exchanges"),
                "enabled": obj.get("enabled"),
                "performance_fee": obj.get("performance_fee"),
            }
        )
        return _obj



# coding: utf-8

# flake8: noqa
"""
Notifications API

API for notification delivery system

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from crypticorn.notification.client.models.broadcast import Broadcast
from crypticorn.notification.client.models.broadcast_create import BroadcastCreate
from crypticorn.notification.client.models.broadcast_update import BroadcastUpdate
from crypticorn.notification.client.models.dashboard_action_button import (
    DashboardActionButton,
)
from crypticorn.notification.client.models.dashboard_field import DashboardField
from crypticorn.notification.client.models.dashboard_notification import (
    DashboardNotification,
)
from crypticorn.notification.client.models.dashboard_notification_update import (
    DashboardNotificationUpdate,
)
from crypticorn.notification.client.models.exception_detail_literal_unknown_error_invalid_data_request_invalid_data_response_object_already_exists_object_not_found_template_not_found_invalid_template_variables import (
    ExceptionDetailLiteralUnknownErrorInvalidDataRequestInvalidDataResponseObjectAlreadyExistsObjectNotFoundTemplateNotFoundInvalidTemplateVariables,
)
from crypticorn.notification.client.models.notification_create import NotificationCreate
from crypticorn.notification.client.models.paginated_response_dashboard_notification import (
    PaginatedResponseDashboardNotification,
)
from crypticorn.notification.client.models.template import Template
from crypticorn.notification.client.models.ui_template import UITemplate
from crypticorn.notification.client.models.user_setting import UserSetting
from crypticorn.notification.client.models.user_setting_create import UserSettingCreate
from crypticorn.notification.client.models.user_setting_update import UserSettingUpdate



import pytest
import pytest_asyncio

from crypticorn import AsyncClient
from crypticorn.hive import Configuration as HiveConfig


@pytest_asyncio.fixture
async def client():
    async with AsyncClient(api_key="test") as client:
        yield client


@pytest.mark.asyncio
async def test_client_config(client: AsyncClient):
    client.configure(config=HiveConfig(host="something"), service="hive")
    assert client.hive.config.host == "something"  # overriden
    assert client.hive.config.api_key == {"APIKeyHeader": "test"}  # not overriden



# coding: utf-8

# flake8: noqa
"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 2.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from crypticorn.trade.client.models.bot import Bot
from crypticorn.trade.client.models.bot_create import BotCreate
from crypticorn.trade.client.models.bot_update import BotUpdate
from crypticorn.trade.client.models.exchange_key import ExchangeKey
from crypticorn.trade.client.models.exchange_key_create import ExchangeKeyCreate
from crypticorn.trade.client.models.exchange_key_update import ExchangeKeyUpdate
from crypticorn.trade.client.models.execution_ids import ExecutionIds
from crypticorn.trade.client.models.futures_trading_action import FuturesTradingAction
from crypticorn.trade.client.models.futures_trading_action_create import (
    FuturesTradingActionCreate,
)
from crypticorn.trade.client.models.order import Order
from crypticorn.trade.client.models.orders_count import OrdersCount
from crypticorn.trade.client.models.paginated_response_futures_trading_action import (
    PaginatedResponseFuturesTradingAction,
)
from crypticorn.trade.client.models.paginated_response_order import (
    PaginatedResponseOrder,
)
from crypticorn.trade.client.models.pn_l import PnL
from crypticorn.trade.client.models.post_futures_action import PostFuturesAction
from crypticorn.trade.client.models.spot_trading_action_create import (
    SpotTradingActionCreate,
)
from crypticorn.trade.client.models.strategy import Strategy
from crypticorn.trade.client.models.strategy_create import StrategyCreate
from crypticorn.trade.client.models.strategy_update import StrategyUpdate
from crypticorn.trade.client.models.tpsl import TPSL
from crypticorn.trade.client.models.tpsl_create import TPSLCreate
from crypticorn.trade.client.models.trading_action_type import TradingActionType
from crypticorn.trade.client.models.vault_bot import VaultBot
from crypticorn.trade.client.models.vault_details import VaultDetails



# coding: utf-8

"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 2.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from typing import Any, Awaitable, Dict, List, Optional, Tuple, Union

from pydantic import Field, StrictFloat, StrictInt, StrictStr, validate_call
from typing_extensions import Annotated

from crypticorn.trade.client.api_client import ApiClient, RequestSerialized
from crypticorn.trade.client.api_response import ApiResponse
from crypticorn.trade.client.models.strategy import Strategy
from crypticorn.trade.client.models.strategy_create import StrategyCreate
from crypticorn.trade.client.models.strategy_update import StrategyUpdate
from crypticorn.trade.client.rest import RESTResponseType

# Import async_to_sync for sync methods
try:
    from asgiref.sync import async_to_sync

    _HAS_ASGIREF = True
except ImportError:
    _HAS_ASGIREF = False

    def async_to_sync(async_func):
        """Fallback decorator that raises an error if asgiref is not available."""

        def wrapper(*args, **kwargs):
            raise ImportError(
                "asgiref is required for sync methods. Install with: pip install asgiref"
            )

        return wrapper


class StrategiesApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None, is_sync: bool = False) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client
        self.is_sync = is_sync

    @validate_call
    def create_strategy(
        self,
        strategy_create: StrategyCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[Strategy, Awaitable[Strategy]]:
        """Create Strategy"""
        if self.is_sync:
            return self._create_strategy_sync(
                strategy_create=strategy_create,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._create_strategy_async(
                strategy_create=strategy_create,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def create_strategy_with_http_info(
        self,
        strategy_create: StrategyCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[ApiResponse[Strategy], Awaitable[ApiResponse[Strategy]]]:
        """Create Strategy with HTTP info"""
        if self.is_sync:
            return self._create_strategy_sync_with_http_info(
                strategy_create=strategy_create,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._create_strategy_async_with_http_info(
                strategy_create=strategy_create,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def create_strategy_without_preload_content(
        self,
        strategy_create: StrategyCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[RESTResponseType, Awaitable[RESTResponseType]]:
        """Create Strategy without preloading content"""
        if self.is_sync:
            return self._create_strategy_sync_without_preload_content(
                strategy_create=strategy_create,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._create_strategy_async_without_preload_content(
                strategy_create=strategy_create,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    # Private async implementation methods
    @validate_call
    async def _create_strategy_async(
        self,
        strategy_create: StrategyCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Strategy:
        """Create Strategy

        Creates a new strategy.

        :param strategy_create: (required)
        :type strategy_create: StrategyCreate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._create_strategy_serialize(
            strategy_create=strategy_create,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "201": "Strategy",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def _create_strategy_async_with_http_info(
        self,
        strategy_create: StrategyCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Strategy]:
        """Create Strategy

        Creates a new strategy.

        :param strategy_create: (required)
        :type strategy_create: StrategyCreate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._create_strategy_serialize(
            strategy_create=strategy_create,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "201": "Strategy",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data, response_types_map=_response_types_map
        )

    @validate_call
    async def _create_strategy_async_without_preload_content(
        self,
        strategy_create: StrategyCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Create Strategy

        Creates a new strategy.

        :param strategy_create: (required)
        :type strategy_create: StrategyCreate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._create_strategy_serialize(
            strategy_create=strategy_create,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "201": "Strategy",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data

    # Private sync implementation methods
    @validate_call
    def _create_strategy_sync(
        self,
        strategy_create: StrategyCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Strategy:
        """Synchronous version of create_strategy"""
        return async_to_sync(self._create_strategy_async)(
            strategy_create=strategy_create,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _create_strategy_sync_with_http_info(
        self,
        strategy_create: StrategyCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Strategy]:
        """Synchronous version of create_strategy_with_http_info"""
        return async_to_sync(self._create_strategy_async_with_http_info)(
            strategy_create=strategy_create,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _create_strategy_sync_without_preload_content(
        self,
        strategy_create: StrategyCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Synchronous version of create_strategy_without_preload_content"""
        return async_to_sync(self._create_strategy_async_without_preload_content)(
            strategy_create=strategy_create,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    def _create_strategy_serialize(
        self,
        strategy_create,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if strategy_create is not None:
            _body_params = strategy_create

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params["Content-Type"] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(
                ["application/json"]
            )
            if _default_content_type is not None:
                _header_params["Content-Type"] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader", "HTTPBearer"]

        return self.api_client.param_serialize(
            method="POST",
            resource_path="/strategies",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def get_strategies(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[List[Strategy], Awaitable[List[Strategy]]]:
        """Get Strategies"""
        if self.is_sync:
            return self._get_strategies_sync(
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_strategies_async(
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_strategies_with_http_info(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[ApiResponse[List[Strategy]], Awaitable[ApiResponse[List[Strategy]]]]:
        """Get Strategies with HTTP info"""
        if self.is_sync:
            return self._get_strategies_sync_with_http_info(
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_strategies_async_with_http_info(
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_strategies_without_preload_content(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[RESTResponseType, Awaitable[RESTResponseType]]:
        """Get Strategies without preloading content"""
        if self.is_sync:
            return self._get_strategies_sync_without_preload_content(
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_strategies_async_without_preload_content(
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    # Private async implementation methods
    @validate_call
    async def _get_strategies_async(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[Strategy]:
        """Get Strategies

        Get all strategies.

        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_strategies_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[Strategy]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def _get_strategies_async_with_http_info(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[Strategy]]:
        """Get Strategies

        Get all strategies.

        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_strategies_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[Strategy]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data, response_types_map=_response_types_map
        )

    @validate_call
    async def _get_strategies_async_without_preload_content(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Strategies

        Get all strategies.

        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_strategies_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[Strategy]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data

    # Private sync implementation methods
    @validate_call
    def _get_strategies_sync(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[Strategy]:
        """Synchronous version of get_strategies"""
        return async_to_sync(self._get_strategies_async)(
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_strategies_sync_with_http_info(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[Strategy]]:
        """Synchronous version of get_strategies_with_http_info"""
        return async_to_sync(self._get_strategies_async_with_http_info)(
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_strategies_sync_without_preload_content(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Synchronous version of get_strategies_without_preload_content"""
        return async_to_sync(self._get_strategies_async_without_preload_content)(
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    def _get_strategies_serialize(
        self,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader", "HTTPBearer"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/strategies",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def update_strategy(
        self,
        id: Annotated[
            StrictStr, Field(description="The ID of the strategy to update.")
        ],
        strategy_update: StrategyUpdate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[Strategy, Awaitable[Strategy]]:
        """Update Strategy"""
        if self.is_sync:
            return self._update_strategy_sync(
                id=id,
                strategy_update=strategy_update,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._update_strategy_async(
                id=id,
                strategy_update=strategy_update,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def update_strategy_with_http_info(
        self,
        id: Annotated[
            StrictStr, Field(description="The ID of the strategy to update.")
        ],
        strategy_update: StrategyUpdate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[ApiResponse[Strategy], Awaitable[ApiResponse[Strategy]]]:
        """Update Strategy with HTTP info"""
        if self.is_sync:
            return self._update_strategy_sync_with_http_info(
                id=id,
                strategy_update=strategy_update,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._update_strategy_async_with_http_info(
                id=id,
                strategy_update=strategy_update,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def update_strategy_without_preload_content(
        self,
        id: Annotated[
            StrictStr, Field(description="The ID of the strategy to update.")
        ],
        strategy_update: StrategyUpdate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[RESTResponseType, Awaitable[RESTResponseType]]:
        """Update Strategy without preloading content"""
        if self.is_sync:
            return self._update_strategy_sync_without_preload_content(
                id=id,
                strategy_update=strategy_update,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._update_strategy_async_without_preload_content(
                id=id,
                strategy_update=strategy_update,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    # Private async implementation methods
    @validate_call
    async def _update_strategy_async(
        self,
        id: Annotated[
            StrictStr, Field(description="The ID of the strategy to update.")
        ],
        strategy_update: StrategyUpdate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Strategy:
        """Update Strategy

        Updates a strategy. If the strategy is being disabled, all bots associated with it will be set to stopping.

        :param id: The ID of the strategy to update. (required)
        :type id: str
        :param strategy_update: (required)
        :type strategy_update: StrategyUpdate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._update_strategy_serialize(
            id=id,
            strategy_update=strategy_update,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "Strategy",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def _update_strategy_async_with_http_info(
        self,
        id: Annotated[
            StrictStr, Field(description="The ID of the strategy to update.")
        ],
        strategy_update: StrategyUpdate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Strategy]:
        """Update Strategy

        Updates a strategy. If the strategy is being disabled, all bots associated with it will be set to stopping.

        :param id: The ID of the strategy to update. (required)
        :type id: str
        :param strategy_update: (required)
        :type strategy_update: StrategyUpdate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._update_strategy_serialize(
            id=id,
            strategy_update=strategy_update,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "Strategy",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data, response_types_map=_response_types_map
        )

    @validate_call
    async def _update_strategy_async_without_preload_content(
        self,
        id: Annotated[
            StrictStr, Field(description="The ID of the strategy to update.")
        ],
        strategy_update: StrategyUpdate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Update Strategy

        Updates a strategy. If the strategy is being disabled, all bots associated with it will be set to stopping.

        :param id: The ID of the strategy to update. (required)
        :type id: str
        :param strategy_update: (required)
        :type strategy_update: StrategyUpdate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._update_strategy_serialize(
            id=id,
            strategy_update=strategy_update,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "Strategy",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data

    # Private sync implementation methods
    @validate_call
    def _update_strategy_sync(
        self,
        id: Annotated[
            StrictStr, Field(description="The ID of the strategy to update.")
        ],
        strategy_update: StrategyUpdate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Strategy:
        """Synchronous version of update_strategy"""
        return async_to_sync(self._update_strategy_async)(
            id=id,
            strategy_update=strategy_update,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _update_strategy_sync_with_http_info(
        self,
        id: Annotated[
            StrictStr, Field(description="The ID of the strategy to update.")
        ],
        strategy_update: StrategyUpdate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Strategy]:
        """Synchronous version of update_strategy_with_http_info"""
        return async_to_sync(self._update_strategy_async_with_http_info)(
            id=id,
            strategy_update=strategy_update,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _update_strategy_sync_without_preload_content(
        self,
        id: Annotated[
            StrictStr, Field(description="The ID of the strategy to update.")
        ],
        strategy_update: StrategyUpdate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Synchronous version of update_strategy_without_preload_content"""
        return async_to_sync(self._update_strategy_async_without_preload_content)(
            id=id,
            strategy_update=strategy_update,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    def _update_strategy_serialize(
        self,
        id,
        strategy_update,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params["id"] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if strategy_update is not None:
            _body_params = strategy_update

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params["Content-Type"] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(
                ["application/json"]
            )
            if _default_content_type is not None:
                _header_params["Content-Type"] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader", "HTTPBearer"]

        return self.api_client.param_serialize(
            method="PUT",
            resource_path="/strategies/{id}",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )



import gc
import time
import warnings

from aiohttp import ClientSession

from crypticorn.client import SyncClient


def test_custom_http_client_injection():
    """Test that custom sessions are accepted but not used in sync mode since we use per-operation sessions."""
    import asyncio

    async def _test():
        custom_session = ClientSession()

        try:
            # For sync client, custom sessions are accepted but not used since we create sessions per operation
            client = SyncClient(http_client=custom_session)

            # In sync mode, services should not have the custom session since sessions are created per operation
            subclient = client._services["hive"]
            # Should be None since sync clients don't persist sessions
            assert subclient.base_client.rest_client.pool_manager is None

            client.close()
        finally:
            await custom_session.close()

    asyncio.run(_test())


def test_lazy_http_client_creation():
    """Test that sync client doesn't create persistent sessions."""
    client = SyncClient()
    assert client._http_client is None

    # _ensure_session does nothing in sync mode
    client._ensure_session()

    # Should still be None since sync clients don't persist sessions
    assert client._http_client is None

    subclient = client._services["hive"]
    # Should be None since sessions are created per operation
    assert subclient.base_client.rest_client.pool_manager is None

    client.close()
    assert client._http_client is None


def test_close_custom_http_client_not_owned():
    """Test that custom sessions are not affected by sync client close."""
    import asyncio

    async def _test():
        custom_session = ClientSession()

        try:
            client = SyncClient(http_client=custom_session)

            client.close()
            # Custom session should still be open since sync client doesn't use it
            assert not custom_session.closed
        finally:
            await custom_session.close()

    asyncio.run(_test())


def test_close_owned_http_client():
    """Test that sync client close doesn't affect non-existent persistent sessions."""
    client = SyncClient()

    # Trigger some operation that would create sessions
    client.hive.status.ping()

    # Should still be None since sessions are created and closed per operation
    assert client._http_client is None

    client.close()
    assert client._http_client is None


def test_no_persistent_sessions_in_sync_mode():
    """Test that sync client creates sessions per operation, not persistent ones."""
    client = SyncClient()

    # Make multiple calls
    for _ in range(3):
        response = client.hive.status.ping()
        assert response is not None
        # Should still be None after each call
        assert client._http_client is None

    # All services should still have None for pool_manager
    subclient = client._services["hive"]
    assert subclient.base_client.rest_client.pool_manager is None

    client.close()


def test_sync_client_operations_work():
    """Test that basic operations work with per-operation session management."""
    client = SyncClient()

    try:
        # Multiple operations should all work
        responses = []
        for _ in range(3):
            responses.append(client.hive.status.ping())

        # All responses should be valid
        assert len(responses) == 3
        for response in responses:
            assert response is not None

        # No persistent session should exist
        assert client._http_client is None

    finally:
        client.close()


def test_context_manager_usage():
    """Test context manager usage with sync client."""
    with SyncClient() as client:
        # Should not have persistent session
        assert client._http_client is None

        # Operations should work
        response = client.hive.status.ping()
        assert response is not None

        # Still no persistent session
        assert client._http_client is None

        subclient = client._services["hive"]
        # Should be None since sessions are per-operation
        assert subclient.base_client.rest_client.pool_manager is None

    # After context manager, client should be clean
    assert client._http_client is None


def test_concurrent_operations():
    """Test that multiple operations work correctly without session conflicts."""
    client = SyncClient()

    try:
        # Run multiple operations that would each create their own session
        responses = []
        for i in range(5):
            response = client.hive.status.ping()
            responses.append(response)
            # Verify no session persistence
            assert client._http_client is None

        # All responses should be valid
        assert len(responses) == 5
        for response in responses:
            assert response is not None

    finally:
        client.close()


def test_no_session_warnings_in_sync_mode():
    """Test that sync client doesn't generate unclosed session warnings."""
    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter("always")

        client = SyncClient()
        client.hive.status.ping()

        # Intentionally don't close manually to test cleanup
        del client
        time.sleep(0.1)  # Let cleanup happen
        gc.collect()
        time.sleep(0.1)

        # Should not have unclosed session warnings since sessions are cleaned per operation
        unclosed_warnings = [
            warn for warn in w if "Unclosed client session" in str(warn.message)
        ]
        # Sync client should not generate these warnings due to per-operation cleanup
        assert len(unclosed_warnings) == 0



from __future__ import annotations

from typing import TYPE_CHECKING, Awaitable, Optional, Union

from crypticorn._internal.utils import optional_import
from crypticorn.metrics import (
    ApiClient,
    Configuration,
    ExchangesApi,
    IndicatorsApi,
    LogsApi,
    MarketcapApi,
    MarketsApi,
    QuoteCurrenciesApi,
    StatusApi,
    TokensApi,
)

try:
    import pandas as pd
except ImportError:
    pd = None

if TYPE_CHECKING:
    from aiohttp import ClientSession


class MetricsClient:
    """
    A client for interacting with the Crypticorn Metrics API.
    """

    config_class = Configuration

    def __init__(
        self,
        config: Configuration,
        http_client: Optional[ClientSession] = None,
        is_sync: bool = False,
    ):
        self.config = config
        self.base_client = ApiClient(configuration=self.config)
        if http_client is not None:
            self.base_client.rest_client.pool_manager = http_client
        # Pass sync context to REST client for proper session management
        self.base_client.rest_client.is_sync = is_sync
        # Instantiate all the endpoint clients
        self.status = StatusApi(self.base_client, is_sync=is_sync)
        self.indicators = IndicatorsApi(self.base_client, is_sync=is_sync)
        self.logs = LogsApi(self.base_client, is_sync=is_sync)
        self.marketcap = MarketcapApiWrapper(self.base_client, is_sync=is_sync)
        self.markets = MarketsApi(self.base_client, is_sync=is_sync)
        self.tokens = TokensApiWrapper(self.base_client, is_sync=is_sync)
        self.exchanges = ExchangesApiWrapper(self.base_client, is_sync=is_sync)
        self.quote_currencies = QuoteCurrenciesApi(self.base_client, is_sync=is_sync)


class MarketcapApiWrapper(MarketcapApi):
    """
    A wrapper for the MarketcapApi class.
    """

    def get_marketcap_symbols_fmt(
        self, *args, **kwargs
    ) -> Union["pd.DataFrame", Awaitable["pd.DataFrame"]]:
        """
        Get the marketcap symbols in a pandas dataframe
        Works in both sync and async contexts.
        """
        if self.is_sync:
            return self._get_marketcap_symbols_fmt_sync(*args, **kwargs)
        else:
            return self._get_marketcap_symbols_fmt_async(*args, **kwargs)

    def _get_marketcap_symbols_fmt_sync(self, *args, **kwargs) -> "pd.DataFrame":
        """
        Get the marketcap symbols in a pandas dataframe (sync version)
        """
        pd = optional_import("pandas", "extra")
        response = self._get_marketcap_symbols_sync(*args, **kwargs)
        rows = []
        for item in response:
            row: dict[Union[int, str], Union[int, str, None]] = {
                "timestamp": item.timestamp
            }
            row.update({i + 1: sym for i, sym in enumerate(item.symbols)})
            rows.append(row)
        df = pd.DataFrame(rows)
        return df

    async def _get_marketcap_symbols_fmt_async(self, *args, **kwargs) -> "pd.DataFrame":
        """
        Get the marketcap symbols in a pandas dataframe (async version)
        """
        pd = optional_import("pandas", "extra")
        response = await self._get_marketcap_symbols_async(*args, **kwargs)
        rows = []
        for item in response:
            row: dict[Union[int, str], Union[int, str, None]] = {
                "timestamp": item.timestamp
            }
            row.update({i + 1: sym for i, sym in enumerate(item.symbols)})
            rows.append(row)
        df = pd.DataFrame(rows)
        return df


class TokensApiWrapper(TokensApi):
    """
    A wrapper for the TokensApi class.
    """

    def get_stable_tokens_fmt(
        self, *args, **kwargs
    ) -> Union["pd.DataFrame", Awaitable["pd.DataFrame"]]:
        """
        Get the tokens in a pandas dataframe
        Works in both sync and async contexts.
        """
        if self.is_sync:
            return self._get_stable_tokens_fmt_sync(*args, **kwargs)
        else:
            return self._get_stable_tokens_fmt_async(*args, **kwargs)

    def _get_stable_tokens_fmt_sync(self, *args, **kwargs) -> "pd.DataFrame":
        """
        Get the tokens in a pandas dataframe (sync version)
        """
        pd = optional_import("pandas", "extra")
        response = self._get_stable_tokens_sync(*args, **kwargs)
        return pd.DataFrame(response)

    async def _get_stable_tokens_fmt_async(self, *args, **kwargs) -> "pd.DataFrame":
        """
        Get the tokens in a pandas dataframe (async version)
        """
        pd = optional_import("pandas", "extra")
        response = await self._get_stable_tokens_async(*args, **kwargs)
        return pd.DataFrame(response)

    def get_wrapped_tokens_fmt(
        self, *args, **kwargs
    ) -> Union["pd.DataFrame", Awaitable["pd.DataFrame"]]:
        """
        Get the wrapped tokens in a pandas dataframe
        Works in both sync and async contexts.
        """
        if self.is_sync:
            return self._get_wrapped_tokens_fmt_sync(*args, **kwargs)
        else:
            return self._get_wrapped_tokens_fmt_async(*args, **kwargs)

    def _get_wrapped_tokens_fmt_sync(self, *args, **kwargs) -> "pd.DataFrame":
        """
        Get the wrapped tokens in a pandas dataframe (sync version)
        """
        pd = optional_import("pandas", "extra")
        response = self._get_wrapped_tokens_sync(*args, **kwargs)
        return pd.DataFrame(response)

    async def _get_wrapped_tokens_fmt_async(self, *args, **kwargs) -> "pd.DataFrame":
        """
        Get the wrapped tokens in a pandas dataframe (async version)
        """
        pd = optional_import("pandas", "extra")
        response = await self._get_wrapped_tokens_async(*args, **kwargs)
        return pd.DataFrame(response)


class ExchangesApiWrapper(ExchangesApi):
    """
    A wrapper for the ExchangesApi class.
    """

    def get_available_exchanges_fmt(
        self, *args, **kwargs
    ) -> Union["pd.DataFrame", Awaitable["pd.DataFrame"]]:
        """
        Get the exchanges in a pandas dataframe
        Works in both sync and async contexts.
        """
        if self.is_sync:
            return self._get_available_exchanges_fmt_sync(*args, **kwargs)
        else:
            return self._get_available_exchanges_fmt_async(*args, **kwargs)

    def _get_available_exchanges_fmt_sync(
        self, *args, **kwargs
    ) -> "pd.DataFrame":  # noqa: F821
        """
        Get the exchanges in a pandas dataframe (sync version)
        """
        pd = optional_import("pandas", "extra")
        response = self._get_available_exchanges_sync(*args, **kwargs)

        # Create list of dictionaries with timestamp and flattened exchange data
        rows = []
        for item in response:
            row: dict[str, Union[int, bool]] = {"timestamp": item.timestamp}
            row.update(
                item.exchanges
            )  # This spreads the exchanges dict into individual columns
            rows.append(row)

        df = pd.DataFrame(rows)
        return df

    async def _get_available_exchanges_fmt_async(
        self, *args, **kwargs
    ) -> "pd.DataFrame":  # noqa: F821
        """
        Get the exchanges in a pandas dataframe (async version)
        """
        pd = optional_import("pandas", "extra")
        response = await self._get_available_exchanges_async(*args, **kwargs)

        # Create list of dictionaries with timestamp and flattened exchange data
        rows = []
        for item in response:
            row: dict[str, Union[int, bool]] = {"timestamp": item.timestamp}
            row.update(
                item.exchanges
            )  # This spreads the exchanges dict into individual columns
            rows.append(row)

        df = pd.DataFrame(rows)
        return df
